clc
%close all
clear all
y=load('y2_y40.dat'); %load the first 100 measurament obtained from time 2 to time 101
y=[0,y]; % to have that y(i) indicates the measurament performed at time i
k=40; %current time
%% state estimation based on noisy measurement

%process model parameters: x(t)=ax(t-1)^2+bx(t-1)+c+omega_proc
a=0.01;
b=1;
c=0.001;
std_omega_proc=0.001;

%measurament model parameters: z(t)=x(t)^2+omega_meas
std_omega_meas=0.05;

% particles sampling

Nsamp = 1000;

%initial weights
w(1,:) = ones(Nsamp,1)/Nsamp;
%initial particles
x(1,:)=rand(1,Nsamp)*0.05; %from uniform [0,0.05]

for t=2:k
    t
    %prediction stage
    x(t,:)=a*(x(t-1,:)).^2+b*x(t-1,:)+c+randn(1,Nsamp)*std_omega_proc;
    %update stage
    for i=1:Nsamp
        lik(t,i)=(1/(sqrt(2*pi)*std_omega_meas))*...                      % likelihood 
        exp(-0.5*(((x(t,i)-y(t))/std_omega_meas)).^2);
    end
    w(t,:)=w(t-1,:).*lik(t,:); %new weight
    w(t,:)=w(t,:)/sum(w(t,:)); %renormalize weight
    
    %estimate the mean and standard deviation of the particles
    x_mean(t)=x(t,:)*w(t,:)';
    x_std(t)=sqrt(sum((x(t,:)-x_mean(t)).^2.*w(t,:))*Nsamp/(Nsamp-1));
    
    %estimate state percentiles
    x_vect=x(t,:);
    [x_sort,pos]=sort(x_vect); %ordering the particles from the smallest to the largest
    w_sort=w(t,pos);  %corresponding weights
    w_cum=cumsum(w_sort); %building cumulative distribution of the state
    ind_perc5=find(w_cum>=0.05,1); 
    x5(t)=x_sort(ind_perc5);
    ind_perc95=find(w_cum>=0.95,1);
    x95(t)=x_sort(ind_perc95);
    
    %resampling (bootstrap, replacement)
    rr = rand(Nsamp,1);
    for s=1:Nsamp
        i_res(s) = find(cumsum(w(t,:))>=rr(s),1,'first'); %find the indexes of the new candidate particle
    end
    x(t,:) = x(t,i_res); %replace the old particles with the new ones
    w(t,:)=ones(Nsamp,1)/Nsamp;
end

figure
plot(x_mean)
hold on
plot(x5,'b');
plot(x95,'k');
title('degradation state mean and percentiles')
hold on
load x_true
plot(x_true,'g')
plot(2:k,y(2:k),'r')
title('degradation state mean and standard deviation')
legend('recon mean','perc 5','perc95','true','meas')
%%
%prediction of the RUL starting from the last updating stage

FT=1; %Failure Threshold

% find RUL distribution, the mean and the standard deviation
for i=1:Nsamp
    t=k; %current time
    while x(t,i)<FT
        t=t+1;
        %prediction stage
        x(t,i)=a*(x(t-1,i)).^2+b*x(t-1,i)+c+randn*std_omega_proc;
    end
    RUL(i)=t-k;
end
RUL_mean = mean(RUL);
RUL_std = std(RUL);

TRUE_RUL = length(x_true)-k;

figure
hist(RUL)

